/**
 * 
 */
package mergesort;

/**
 * @author meder
 * 
 */
public class ParallelMergeSort extends MergeSort {
	private static final int MIN_SIZE_FOR_PARALLEL_SORT = 8;

	private static final SequentialMergeSort SEQUENTIAL_INST = new SequentialMergeSort();

	/**
	 * @param array
	 *            array to sort
	 * @return sorted array
	 */
	@Override
	public int[] sort(int[] array) {
		MergeSortWorker msw = new MergeSortWorker(array);
		Thread master = new Thread(msw);
		master.start();

		try {
			master.join();
		} catch (InterruptedException e) {
			e.printStackTrace();
		}

		return msw.getSortedArray();
	}

	/**
	 * @author meder
	 */
	private class MergeSortWorker implements Runnable {

		private int[] array = null;

		/**
		 * @param array
		 *            array to sort
		 */
		public MergeSortWorker(int[] array) {
			this.array = array;
		}

		/**
		 * 
		 */
		@Override
		public void run() {
			// Use parallel algorithm, if array length is greater than
			// MIN_SIZE_FOR_PARALLEL_SORT
			if (MIN_SIZE_FOR_PARALLEL_SORT < array.length) {

				if (array.length <= 1) {
					return;
				}

				SplitArray s = split(array);

				int[] part1 = s.getPart1();
				int[] part2 = s.getPart2();

				// Create two worker threads
				MergeSortWorker msw1 = new MergeSortWorker(part1);
				MergeSortWorker msw2 = new MergeSortWorker(part2);
				Thread worker1 = new Thread(msw1);
				Thread worker2 = new Thread(msw2);

				// Start the two worker threads
				worker1.start();
				worker2.start();
				try {
					// Wait, until workers have finished
					worker1.join();
					worker2.join();

				} catch (InterruptedException e) {
					e.printStackTrace();
				}

				// Merge the sorted sub-arrays
				array = merge(msw1.getSortedArray(), msw2.getSortedArray());

			} else {
				array = SEQUENTIAL_INST.sort(array);
			}

		}

		public int[] getSortedArray() {
			return array;
		}
	}

}
